///////////////////////////////////////////////////////////////////////////////
//
// Impulse-C(c) 2003-2008 Impulse Accelerated Technologies, Inc.
//
// FPWM_hw.c: includes the hardware processes and configuration
// function for the basic floating-point version of the FPWM
// system.
//
// See additional comments in FPWM.h.
//

#include "co.h"
#include "cosim_log.h"
#include "FPWM.h"
#include "co_math.h"

// Software process declarations (see FPWM_sw.c)
extern void Producer(co_stream threshold_stream, co_stream pwm_stream, co_stream iteration_stream, co_stream dna_stream, 
	co_parameter filter);
extern void Consumer(co_stream start_stream, co_stream score_stream);


//
// This is the hardware 'pwm' process
//
void PWM(co_stream pwm_stream, co_stream iteration_stream, co_stream dna_stream, co_stream counter_stream, co_stream sum_stream)
{
	int i, j, k, l;
	float nSamplePwm;
	float pwm[COLUMNS][ROWS];
	int16 dnaLength;
	int motifLength;
	char nSampleDna;
	char sequence[MAX_SEQUENCE];
	int16 counter;
	float sum;
	int16 nHitStart;
	float nHitScore;
	
	IF_SIM(int samplesread; int resultswritten;)

	IF_SIM(cosim_logwindow log;)
	IF_SIM(log = cosim_logwindow_create("PWM");)

	do {	// Hardware processes run forever
		IF_SIM(samplesread=0; resultswritten=0;)
		
		// Stating motif length
		motifLength = ROWS;

		// Opening streams
		co_stream_open(pwm_stream, O_RDONLY, FLOAT_TYPE);
		co_stream_open(iteration_stream, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(dna_stream, O_RDONLY, CHAR_TYPE);
		co_stream_open(counter_stream, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(sum_stream, O_WRONLY, FLOAT_TYPE);

		// Reading pwm from stream
		for(j=0; j<ROWS; j++) {
			for(i=0; i<COLUMNS; i++) {
				co_stream_read(pwm_stream, &nSamplePwm, sizeof(float));
				pwm[i][j] = (float)nSamplePwm;
			}
		}
		
		// Reading sequence length from stream
		if(co_stream_read(iteration_stream, &dnaLength, sizeof(int16)) == co_err_none) {
			IF_SIM(cosim_logwindow_fwrite(log, "Sequence length is %d\n", dnaLength);)
		}

		// Reading portion of sequence from stream neccessary to start computation
		for(k = 0; k < motifLength; k++) {
			co_stream_read(dna_stream, &nSampleDna, sizeof(char));
			IF_SIM(samplesread++;)
			sequence[k] = (char)nSampleDna;
		}

		// Initiating counter and stream
		counter = 0;
		sum = 0;

		// Computing result scores for all subsequences
		while(counter <= (dnaLength - motifLength)) {

			// Calculating score for current position
			for (l = 0; l < motifLength; l++) {
			#pragma CO PIPELINE
			#pragma CO SET stageDelay 32
				switch(sequence[l+counter]) {
					case((char)'A'):
						sum += pwm[0][l];
						break;
					case((char)'C'):
						sum += pwm[1][l];
						break;
					case((char)'G'):
						sum += pwm[2][l];
						break;
					case((char)'T'):
						sum += pwm[3][l];
						break;
					default:
						// Do nothing here
						break;
				}
			}

			// Stating that counter and sum will be result data
			nHitStart = counter;
			nHitScore = sum;

			// Sending result to filter
			co_stream_write(counter_stream, &nHitStart, sizeof(int16));
			co_stream_write(sum_stream, &nHitScore, sizeof(float));

			IF_SIM(resultswritten++;)
			IF_SIM(cosim_logwindow_fwrite(log, "Wrote score %f to filter, for pattern starting at position %d.\n", nHitScore, nHitStart);)
		
			// Reading new sequence base from stream
			if(co_stream_read(dna_stream, &nSampleDna, sizeof(char)) == co_err_none) {
				IF_SIM(samplesread++;)
				sequence[k] = (char)nSampleDna;
				k++;
			} else {
				break;
			}
			
			// Updating counter and reseting sum
			counter++;
			sum = 0;
		}
		
		// Closing streams
		co_stream_close(pwm_stream);
		co_stream_close(iteration_stream);
		co_stream_close(dna_stream);
		co_stream_close(counter_stream);
		co_stream_close(sum_stream);

		IF_SIM(cosim_logwindow_fwrite(log, "Closing PWM process; Symbols in sequence read: %d, results written: %d\n", samplesread, resultswritten);)
		IF_SIM(break;)	// Only run once for desktop simulation
	} while(1);
}

//
// This is the hardware 'filter' process
//
void Filter(co_stream threshold_stream, co_stream counter_stream, co_stream sum_stream, co_stream start_stream, co_stream score_stream)
{	
	float threshold;
	int filterMode;
	int16 nResultStart;
	float nResultScore;
	int16 nHitStart;
	float nHitScore;
	IF_SIM(int resultsread; int resultswritten;)

	IF_SIM(cosim_logwindow log;)
	IF_SIM(log = cosim_logwindow_create("Filter");)

	do {	// Hardware processes run forever
		IF_SIM(resultsread=0; resultswritten=0;)
		
		// Opening streams
		co_stream_open(threshold_stream, O_RDONLY, FLOAT_TYPE);
		co_stream_open(counter_stream, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(sum_stream, O_RDONLY, FLOAT_TYPE);
		co_stream_open(start_stream, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(score_stream, O_WRONLY, FLOAT_TYPE);

		// Reading threshold value from stream; determining filter mode
		if(co_stream_read(threshold_stream, &threshold, sizeof(float)) != co_err_none) {
			// Filter is set to summation filter
			filterMode = 1;
			IF_SIM(cosim_logwindow_fwrite(log, "Filter Mode: %d\n", filterMode);)
		} else {
			// Filter is set to threshold filter
			filterMode = 2;
			IF_SIM(cosim_logwindow_fwrite(log, "Filter Mode: %d\n", filterMode);)
			IF_SIM(cosim_logwindow_fwrite(log, "Threshold value: %f\n", threshold);)
		}

		// Initiating filtered result data
		nResultStart = 0;
		nResultScore = 0;

		// Reading computed results from stream
		while(co_stream_read(counter_stream, &nHitStart, sizeof(int16)) == co_err_none) {		
			if(co_stream_read(sum_stream, &nHitScore, sizeof(float)) == co_err_none) {
				IF_SIM(resultsread++;)

				if(filterMode == 1) {	// Summation filter
					nResultScore += nHitScore;
				} else {	// Threshold filter
					if(nHitScore >= threshold) {
						// Stating that the result score will pass through the filter
						nResultStart = nHitStart;
						nResultScore = nHitScore;
			
						// Sending filtered result to consumer
						co_stream_write(start_stream, &nResultStart, sizeof(int16));
						co_stream_write(score_stream, &nResultScore, sizeof(float));

						IF_SIM(resultswritten++;)
						IF_SIM(cosim_logwindow_fwrite(log, "Filtered score %f for pattern starting at position %d.\n", nResultScore, nResultStart);)
					}
				}
			}
		}

		if(filterMode == 1) {	// Summation filter	
			// Sending result score from summation filter to consumer
			co_stream_write(start_stream, &nResultStart, sizeof(int16));
			co_stream_write(score_stream, &nResultScore, sizeof(float));

			IF_SIM(resultswritten++;)
			IF_SIM(cosim_logwindow_fwrite(log, "Filtered combined score %f for the entire sequence.\n", nResultScore);)
		}
		
		// Closing streams
		co_stream_close(threshold_stream);
		co_stream_close(counter_stream);
		co_stream_close(sum_stream);
		co_stream_close(start_stream);
		co_stream_close(score_stream);

		IF_SIM(cosim_logwindow_fwrite(log,
			   "Closing Filter process; Results read: %d, results filtered: %d\n", resultsread, resultswritten);)

		IF_SIM(break;)	// Only run once for desktop simulation
	} while(1);
}


//
// Impulse C configuration function
//
void config_FPWM(void *arg)
{	
	co_stream threshold_stream;
	co_stream pwm_stream;
	co_stream iteration_stream;
	co_stream dna_stream;
	co_stream counter_stream;
	co_stream sum_stream;
	co_stream start_stream;
	co_stream score_stream;

	co_process producer_process;
	co_process pwm_process;
	co_process filter_process;
	co_process consumer_process;
	IF_SIM(cosim_logwindow_init();)

	char *parameter;

	if (arg != NULL) {
		parameter = (char*) arg;
	} else {
		parameter = NULL;
	}

	threshold_stream = co_stream_create("threshold_stream", FLOAT_TYPE, MIN_STREAMDEPTH);
	pwm_stream = co_stream_create("pwm_stream", FLOAT_TYPE, COLUMNS*ROWS);
	iteration_stream = co_stream_create("iteration_stream", INT_TYPE(MAX_STREAMWIDTH/4), MIN_STREAMDEPTH);
	dna_stream = co_stream_create("dna_stream", CHAR_TYPE, MAX_SEQUENCE);
	counter_stream = co_stream_create("counter_stream", INT_TYPE(MAX_STREAMWIDTH/4), FILTER_QUEUE);
	sum_stream = co_stream_create("sum_stream", FLOAT_TYPE, FILTER_QUEUE);
	start_stream = co_stream_create("start_stream", INT_TYPE(MAX_STREAMWIDTH/4), WRITE_QUEUE);
	score_stream = co_stream_create("score_stream", FLOAT_TYPE, WRITE_QUEUE);

	producer_process = co_process_create("Producer", (co_function)Producer,
										5,
										threshold_stream,
										pwm_stream,
										iteration_stream,
										dna_stream,
										parameter);

	pwm_process = co_process_create("PWM", (co_function)PWM,
									5,
									pwm_stream,
									iteration_stream,
									dna_stream,
									counter_stream,
									sum_stream);

	filter_process = co_process_create("Filter", (co_function)Filter,
									5,
									threshold_stream,
									counter_stream,
									sum_stream,
									start_stream,
									score_stream);

	consumer_process = co_process_create("Consumer",(co_function)Consumer,
										2,
										start_stream,
										score_stream);

	co_process_config(pwm_process, co_loc, "PE0");
	co_process_config(filter_process, co_loc, "PE0");
}

co_architecture co_initialize(int param)
{
	return(co_architecture_create("FPWM","generic_vhdl",config_FPWM,(void *)param));
}
