/*********************************************************************
* Copyright (c) 2005, Impulse Accelerated Technologies, Inc.
* All Rights Reserved.
*
* co_stream.c: Stream functions.
*
*********************************************************************/

#include "einlib.h"

#include <malloc.h>
#include <string.h>

#include "co.h"

// FPGA register addresses
#define DATA_OFFSET		(0*sizeof(uint64))
#define STATUS_OFFSET	(1*sizeof(uint64))
#define CLOSE_OFFSET	(2*sizeof(uint64))

// Status bits
#define READY_MASK	0x1
#define EOS_MASK	0x2

extern int _co_fpga_id;

co_stream co_stream_create(const char * name, co_type datatype, unsigned int buffer_depth)
{
	co_stream stream = malloc(sizeof(co_stream_t));	
	if ( stream != NULL ) {
		stream->name = strdup(name);
		stream->datatype = datatype;
		stream->buffer_depth = buffer_depth;
		stream->direction = CO_NO_DIRECTION;
		stream->io = NULL;
	}

	return stream;
}

void co_stream_attach(co_stream stream, void * io, co_stream_direction dir)
{
	if ( stream != NULL ) {
		stream->io = io;
		stream->direction = dir;
	}
}

co_error co_stream_close(co_stream stream) 
{
	err_e e;

	if ( stream->direction == HW_OUTPUT ) return co_err_none;

	if ( stream != NULL ) {
		fpga_wrt_appif_val(_co_fpga_id, 1, (unsigned long)stream->io + CLOSE_OFFSET, 0, &e);
	}

	return co_err_none;
}

int co_stream_eos(co_stream stream)
{
	uint64 status;
	err_e e;
	
	fpga_rd_appif_val(_co_fpga_id, &status, (unsigned long)stream->io + STATUS_OFFSET, &e);
	
	return ((status & EOS_MASK) != 0) || (status == 0);
}

co_error co_stream_read(co_stream stream, void * buffer, unsigned int buffersize)
{
	uint64 data, status;
	err_e e;

	if ( stream == NULL ) {
		return co_err_invalid_arg;
	}

	do {  // Block until ready
		fpga_rd_appif_val(_co_fpga_id, &status, (unsigned long)stream->io + STATUS_OFFSET, &e);
	} while ( (status & READY_MASK) != 1 && (status & EOS_MASK) == 0 );
    
	if ( (status & EOS_MASK) != 0 ) {
		return co_err_eos;
	}
	else {
		fpga_rd_appif_val(_co_fpga_id, &data, (unsigned long)stream->io + DATA_OFFSET, &e);
		switch ( buffersize ) {
		case 1:
			*(uint8 *)buffer = data;
			break;
		case 2:
			*(uint16 *)buffer = data;
			break;
		case 3:
		case 4:
			*(uint32 *)buffer = data;
			break;
		default: 
			*(uint64 *)buffer = data;
			break;
		}
	}

	return co_err_none;
}

int co_stream_read_nb(co_stream stream, void *buffer, unsigned int buffersize)
{
	uint64 data, status;
	err_e e;
    
	if ( stream == NULL ) {
		return 0;
	}

	fpga_rd_appif_val(_co_fpga_id, &status, (unsigned long)stream->io + STATUS_OFFSET, &e);
    
	if ( (status & READY_MASK) != 1 || (status & EOS_MASK) != 0 ) {
		return 0;
	}
	else {
		fpga_rd_appif_val(_co_fpga_id, &data, (unsigned long)stream->io + DATA_OFFSET, &e);
		switch ( buffersize ) {
		case 1:
			*(uint8 *)buffer = data;
			break;
		case 2:
			*(uint16 *)buffer = data;
			break;
		case 3:
		case 4:
			*(uint32 *)buffer = data;
			break;
		default: 
			*(uint64 *)buffer = data;
			break;
		}
	}

    return 1;
}

co_error co_stream_write(co_stream stream, const void * buffer, unsigned int buffersize)
{
	uint64 data, status;
	err_e e;

	if ( stream == NULL ) {
		return co_err_invalid_arg;
	}

	switch ( buffersize ) {
	case 1:
		data = *(uint8 *)buffer;
		break;
	case 2:
		data = *(uint16 *)buffer;
		break;
	case 3:
	case 4:
		data = *(uint32 *)buffer;
		break;
	default:
		data = *(uint64 *)buffer;
		break;
	}

	do {  // Block until ready
		fpga_rd_appif_val(_co_fpga_id, &status, (unsigned long)stream->io + STATUS_OFFSET, &e);
	} while ( (status & READY_MASK) != 1 );
	
	fpga_wrt_appif_val(_co_fpga_id, data, (unsigned long)stream->io + DATA_OFFSET, 0, &e);

	return co_err_none;
}

co_error co_stream_free(co_stream stream)
{
	if ( stream != NULL ) {
		free(stream->name);
		free(stream);
	}
	
	return co_err_none;
}

