///////////////////////////////////////////////////////////////////////////////
//
// Impulse-C(c) 2003-2008 Impulse Accelerated Technologies, Inc.
//
// FPWM2008_hw.c: includes the hardware processes and configuration
// function for the parallel floating-point version of the FPWM
// system.
//
// See additional comments in FPWM2008.h.
//

#include "co.h"
#include "cosim_log.h"
#include "FPWM2008.h"
#include "co_math.h"

// Software process declarations (see FPWM2008_sw.c)
extern void Producer(co_stream threshold_0, co_stream threshold_1, co_stream pwm_0, co_stream pwm_1, co_stream iteration_0, 
	co_stream iteration_1, co_stream dna_0, co_stream dna_1, co_parameter filters);
extern void Consumer(co_stream start_0, co_stream score_0, co_stream start_1, co_stream score_1);


//
// This is the hardware 'pwm' process
//
void PWM(co_stream pwm, co_stream iteration, co_stream dna, co_stream counter, co_stream sum, co_parameter nInstance)
{
	int i, j, k, l;
	float nSamplePwm;
	float matrix[COLUMNS][ROWS];
	int16 dnaLength;
	int motifLength;
	char nSampleDna;
	char sequence[MAX_SEQUENCE];
	int16 nCounter;
	float nSum;
	int16 nHitStart;
	float nHitScore;
	
	IF_SIM(int samplesread; int resultswritten;)

	IF_SIM(cosim_logwindow log;)
	IF_SIM(log = cosim_logwindow_create("PWM");)

	do {	// Hardware processes run forever
		IF_SIM(samplesread=0; resultswritten=0;)
		
		// Stating motif length
		motifLength = ROWS;
		
		// Opening streams
		co_stream_open(pwm, O_RDONLY, FLOAT_TYPE);
		co_stream_open(iteration, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(dna, O_RDONLY, CHAR_TYPE);
		co_stream_open(counter, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(sum, O_WRONLY, FLOAT_TYPE);

		// Reading pwm from stream
		for(j=0; j<ROWS; j++) {
			for(i=0; i<COLUMNS; i++) {
				co_stream_read(pwm, &nSamplePwm, sizeof(float));
				matrix[i][j] = (float)nSamplePwm;
			}
		}

		// Reading sequence length from stream
		if(co_stream_read(iteration, &dnaLength, sizeof(int16)) == co_err_none) {
			IF_SIM(cosim_logwindow_fwrite(log, "Sequence length is %d\n", dnaLength);)
		}

		// Reading portion of sequence from stream neccessary to start computation
		for(k = 0; k < motifLength; k++) {
			co_stream_read(dna, &nSampleDna, sizeof(char));
			IF_SIM(samplesread++;)
			sequence[k] = (char)nSampleDna;
		}

		// Initiating counter and sum
		nCounter = 0;
		nSum = 0;

		// Computing result scores for all subsequences
		while(nCounter <= (dnaLength - motifLength)) {
			
			// Calculating score for current position
			for (l = 0; l < motifLength; l++) {
			#pragma CO PIPELINE
			#pragma CO SET stageDelay 32
				switch(sequence[l+nCounter]) {
					case((char)'A'):
						nSum += matrix[0][l];
						break;
					case((char)'C'):
						nSum += matrix[1][l];
						break;
					case((char)'G'):
						nSum += matrix[2][l];
						break;
					case((char)'T'):
						nSum += matrix[3][l];
						break;
					default:
						// Do nothing here.
						break;
				}
			}

			// Stating that counter and sum will be result data
			nHitStart = nCounter;
			nHitScore = nSum;

			// Sending result to filter
			co_stream_write(counter, &nHitStart, sizeof(int16));
			co_stream_write(sum, &nHitScore, sizeof(float));

			IF_SIM(resultswritten++;)
			IF_SIM(cosim_logwindow_fwrite(log, "Wrote score %f to filter, for pattern starting at position %d.\n", nHitScore, nHitStart);)
		
			// Reading new sequence base from stream
			if(co_stream_read(dna, &nSampleDna, sizeof(char)) == co_err_none) {
				IF_SIM(samplesread++;)
				sequence[k] = (char)nSampleDna;
				k++;
			} else {
				break;
			}
			
			// Updating counter and reseting sum
			nCounter++;
			nSum = 0;
		}

		// Closing streams
		co_stream_close(pwm);
		co_stream_close(iteration);
		co_stream_close(dna);
		co_stream_close(counter);
		co_stream_close(sum);

		IF_SIM(cosim_logwindow_fwrite(log, "Closing PWM process %d; Symbols in sequence read: %d, results written: %d\n", nInstance, samplesread, resultswritten);)
		IF_SIM(break;)	// Only run once for desktop simulation
	} while(1);
}

//
// This is the hardware 'filter' process
//
void Filter(co_stream threshold, co_stream counter, co_stream sum, co_stream start, co_stream score, co_parameter nInstance)
{	
	float nThreshold;
	int filterMode;
	int16 nResultStart;
	float nResultScore;
	int16 nHitStart;
	float nHitScore;
	IF_SIM(int resultsread; int resultswritten;)

	IF_SIM(cosim_logwindow log;)
	IF_SIM(log = cosim_logwindow_create("Filter");)

	do {	// Hardware processes run forever
		IF_SIM(resultsread=0; resultswritten=0;)

		// Opening streams
		co_stream_open(threshold, O_RDONLY, FLOAT_TYPE);
		co_stream_open(counter, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(sum, O_RDONLY, FLOAT_TYPE);
		co_stream_open(start, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));
		co_stream_open(score, O_WRONLY, FLOAT_TYPE);

		// Reading threshold value from stream; determining filter mode
		if(co_stream_read(threshold, &nThreshold, sizeof(float)) != co_err_none) {
			// Filter is set to summation filter
			filterMode = 1;
			IF_SIM(cosim_logwindow_fwrite(log, "Filter Mode: %d\n", filterMode);)
		} else {
			// Filter is set to threshold filter
			filterMode = 2;
			IF_SIM(cosim_logwindow_fwrite(log, "Filter Mode: %d\n", filterMode);)
			IF_SIM(cosim_logwindow_fwrite(log, "Threshold value: %f\n", nThreshold);)
		}

		// Initiating filtered result data
		nResultStart = 0;
		nResultScore = 0;

		// Read computed results from stream
		while(co_stream_read(counter, &nHitStart, sizeof(int16)) == co_err_none) {
			if(co_stream_read(sum, &nHitScore, sizeof(float)) == co_err_none) {
				IF_SIM(resultsread++;)

				if(filterMode == 1) {	// Summation filter
					nResultScore += nHitScore;
				} else {	// Threshold filter
					if(nHitScore >= nThreshold) {
						// Stating that the result score will pass through the filter
						nResultStart = nHitStart;
						nResultScore = nHitScore;
			
						// Sending filtered result to consumer
						co_stream_write(start, &nResultStart, sizeof(int16));
						co_stream_write(score, &nResultScore, sizeof(float));

						IF_SIM(resultswritten++;)	
						IF_SIM(cosim_logwindow_fwrite(log, "Filtered score %f for pattern starting at position %d.\n", nResultScore, nResultStart);)
					}
				}
			}
		}

		if(filterMode == 1) {	// Summation filter
			// Sending result score from summation filter to consumer
			co_stream_write(start, &nResultStart, sizeof(int16));
			co_stream_write(score, &nResultScore, sizeof(float));

			IF_SIM(resultswritten++;)
			IF_SIM(cosim_logwindow_fwrite(log, "Filtered combined score %f for the entire sequence.\n", nResultScore);)
		}
		
		// Closing streams
		co_stream_close(threshold);
		co_stream_close(counter);
		co_stream_close(sum);
		co_stream_close(start);
		co_stream_close(score);

		IF_SIM(cosim_logwindow_fwrite(log,
			   "Closing filter process %d; Results read: %d, results filtered: %d\n", nInstance, resultsread, resultswritten);)

		IF_SIM(break;)	// Only run once for desktop simulation
	} while(1);
}


//
// Impulse C configuration function
//
void config_FPWM2008(void *arg)
{
	co_stream threshold_0;
	co_stream threshold_1;
	co_stream pwm_0;
	co_stream pwm_1;
	co_stream iteration_0;
	co_stream iteration_1;
	co_stream dna_0;
	co_stream dna_1;
	co_stream counter_0;
	co_stream sum_0;
	co_stream counter_1;
	co_stream sum_1;
	co_stream start_0;
	co_stream score_0;
	co_stream start_1;
	co_stream score_1;

	co_process producer_process;
	co_process PWM0;
	co_process PWM1;
	co_process Filter0;
	co_process Filter1;
	co_process consumer_process;
	IF_SIM(cosim_logwindow_init();)

	char *parameters;

	if (arg != NULL) {
		parameters = (char*) arg;
	} else {
		parameters = NULL;
	}

	threshold_0 = co_stream_create("threshold_0", FLOAT_TYPE, MIN_STREAMDEPTH);
	threshold_1 = co_stream_create("threshold_1", FLOAT_TYPE, MIN_STREAMDEPTH);
	pwm_0 = co_stream_create("pwm_0", FLOAT_TYPE, COLUMNS*ROWS);
	pwm_1 = co_stream_create("pwm_1", FLOAT_TYPE, COLUMNS*ROWS);
	iteration_0 = co_stream_create("iteration_0", INT_TYPE(MAX_STREAMWIDTH/4), MIN_STREAMDEPTH);
	iteration_1 = co_stream_create("iteration_1", INT_TYPE(MAX_STREAMWIDTH/4), MIN_STREAMDEPTH);
	dna_0 = co_stream_create("dna_0", CHAR_TYPE, MAX_SEQUENCE);
	dna_1 = co_stream_create("dna_1", CHAR_TYPE, MAX_SEQUENCE);
	counter_0 = co_stream_create("counter_0", INT_TYPE(MAX_STREAMWIDTH/4), FILTER_QUEUE);
	sum_0 = co_stream_create("sum_0", FLOAT_TYPE, FILTER_QUEUE);
	counter_1 = co_stream_create("counter_1", INT_TYPE(MAX_STREAMWIDTH/4), FILTER_QUEUE);
	sum_1 = co_stream_create("sum_1", FLOAT_TYPE, FILTER_QUEUE);
	start_0 = co_stream_create("start_0", INT_TYPE(MAX_STREAMWIDTH/4), WRITE_QUEUE);
	score_0 = co_stream_create("score_0", FLOAT_TYPE, WRITE_QUEUE);
	start_1 = co_stream_create("start_1", INT_TYPE(MAX_STREAMWIDTH/4), WRITE_QUEUE);
	score_1 = co_stream_create("score_1", FLOAT_TYPE, WRITE_QUEUE);

	producer_process = co_process_create("Producer", (co_function)Producer,
										9,
										threshold_0,
										threshold_1,
										pwm_0,
										pwm_1,
										iteration_0,
										iteration_1,
										dna_0,
										dna_1,
										parameters);

	PWM0 = co_process_create("PWM0", (co_function)PWM,
									6,
									pwm_0,
									iteration_0,
									dna_0,
									counter_0,
									sum_0,
									0);

	PWM1 = co_process_create("PWM1", (co_function)PWM,
									6,
									pwm_1,
									iteration_1,
									dna_1,
									counter_1,
									sum_1,
									1);

	Filter0 = co_process_create("Filter0", (co_function)Filter,
									6,
									threshold_0,
									counter_0,
									sum_0,
									start_0,
									score_0,
									0);

	Filter1 = co_process_create("Filter1", (co_function)Filter,
									6,
									threshold_1,
									counter_1,
									sum_1,
									start_1,
									score_1,
									1);

	consumer_process = co_process_create("Consumer",(co_function)Consumer,
										4,
										start_0,
										score_0,
										start_1,
										score_1);

	co_process_config(PWM0, co_loc, "PE0");
	co_process_config(PWM1, co_loc, "PE0");
	co_process_config(Filter0, co_loc, "PE0");
	co_process_config(Filter1, co_loc, "PE0");
}

co_architecture co_initialize(int param)
{
	return(co_architecture_create("FPWM2008","cray_rt",config_FPWM2008,(void *)param));
}

