-- Generate an Impulse C acknowledge signal from floating_point core's rdy
-- signal.  The rdy signal is a single cycle pulse, but Impulse C requires that
-- the acknowledge be held active.

library ieee;
use ieee.std_logic_1164.all;
entity impulse_acknowledge is
  port (
    clk : in  std_ulogic;
    req : in  std_ulogic;
    rdy : in  std_ulogic;
    ack : out std_ulogic);
end impulse_acknowledge;

architecture rtl of impulse_acknowledge is
  signal complete : std_ulogic;
begin
  genack: process (clk)
  begin  -- process genack
    if clk'event and clk = '1' then  -- rising clock edge
      if (req = '1') then
        complete <= '0'; -- new request, clear the complete status
      elsif (rdy = '1') then
        complete <= '1'; -- got result, set the complete status
      end if;
    end if;
  end process genack;

  ack <= complete;
end rtl;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fadd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
end fadd_ll;

architecture fadd_ll_a of fadd_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fadd : floating_point_v2_0
    generic map(
      c_has_add => 1,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result <= std_ulogic_vector(result_slv);
  
end fadd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fsub_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
end fsub_ll;

architecture fsub_ll_a of fsub_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fsub : floating_point_v2_0
    generic map(
      c_has_subtract => 1,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result <= std_ulogic_vector(result_slv);
  
end fsub_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fmul_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
end fmul_ll;

architecture fmul_ll_a of fmul_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;

  xilinx_fmul : floating_point_v2_0
    generic map(
      c_has_multiply => 1,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,    
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result <= std_ulogic_vector(result_slv);
  
end fmul_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fdiv_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    done: out std_ulogic);
end fdiv_ll;

architecture fdiv_ll_a of fdiv_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy, rfd : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fdiv : floating_point_v2_0
    generic map(
      c_b_fraction_width => 24,
      c_has_operation_nd => 1,
      c_a_fraction_width => 24,
      c_latency => 26,
      c_has_rdy => 1,
      c_result_fraction_width => 24,
      c_has_divide => 1,
      c_rate => 26,
      c_has_operation_rfd => 1,
      c_result_width => 32,
      c_b_width => 32,
      c_a_width => 32)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      operation_rfd => rfd,
      clk => clk,
      result => result_slv,
      rdy => rdy);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  process (clk)
  begin  -- process
    if clk'event and clk = '1' then  -- rising clock edge
      if (rdy = '1') then
        result <= std_ulogic_vector(result_slv);
      end if;
    end if;
  end process;
  
  ack0: entity work.impulse_acknowledge port map (clk, go, rdy, done);
  
end fdiv_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fsqrt_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    done: out std_ulogic);
end fsqrt_ll;

architecture fsqrt_ll_a of fsqrt_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy, rfd : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fsqrt : floating_point_v2_0
    generic map(
      c_has_operation_nd => 1,
      c_a_fraction_width => 24,
      c_latency => 25,
      c_has_rdy => 1,
      c_result_fraction_width => 24,
      c_has_sqrt => 1,
      c_rate => 25,
      c_has_operation_rfd => 1,
      c_result_width => 32,
      c_a_width => 32)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      operation_rfd => rfd,
      clk => clk,
      result => result_slv,
      rdy => rdy);

  a_slv <= std_logic_vector(a);
  process (clk)
  begin  -- process
    if clk'event and clk = '1' then  -- rising clock edge
      if (rdy = '1') then
        result <= std_ulogic_vector(result_slv);
      end if;
    end if;
  end process;
  
  ack0: entity work.impulse_acknowledge port map (clk, go, rdy, done);
  
end fsqrt_ll_a;

library ieee;
use ieee.std_logic_1164.all;

entity fneg_ll is
  port (
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(31 downto 0));
end fneg_ll;

architecture fneg_ll_a of fneg_ll is
begin
  result(31) <= a(31) xor '1';
  result(30 downto 0) <= a(30 downto 0);
end fneg_ll_a;

library ieee;
use ieee.std_logic_1164.all;

entity fabs_ll is
  port (
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(31 downto 0));
end fabs_ll;

architecture fabs_ll_a of fabs_ll is
begin
  result(31) <= '0';
  result(30 downto 0) <= a(30 downto 0);
end fabs_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity feq_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end feq_ll;

architecture feq_ll_a of feq_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_feq : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PT_EQUAL,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);
  
end feq_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fneq_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end fneq_ll;

architecture fneq_ll_a of fneq_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fneq : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PT_NOT_EQUAL,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);
  
end fneq_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity flt_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end flt_ll;

architecture flt_ll_a of flt_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_flt : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PT_LESS_THAN,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);
  
end flt_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity flteq_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end flteq_ll;

architecture flteq_ll_a of flteq_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal b_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_flteq : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PRT_LESS_THAN_OR_EQUAL,
      c_a_width => 32,
      c_a_fraction_width => 24,
      c_b_width => 32,
      c_b_fraction_width => 24,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);

end flteq_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity faddd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
end faddd_ll;

architecture faddd_ll_a of faddd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fadd : floating_point_v2_0
    generic map(
      c_has_add => 1,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 4)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result <= std_ulogic_vector(result_slv);
  
end faddd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fsubd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
end fsubd_ll;

architecture fsubd_ll_a of fsubd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fsub : floating_point_v2_0
    generic map(
      c_has_subtract => 1,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 4)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result <= std_ulogic_vector(result_slv);
  
end fsubd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fmuld_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
end fmuld_ll;

architecture fmuld_ll_a of fmuld_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;

  xilinx_fmul : floating_point_v2_0
    generic map(
      c_has_multiply => 1,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 4)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result <= std_ulogic_vector(result_slv);
  
end fmuld_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fdivd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    done: out std_ulogic);
end fdivd_ll;

architecture fdivd_ll_a of fdivd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy, rfd : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fdiv : floating_point_v2_0
    generic map(
      c_b_fraction_width => 53,
      c_has_operation_nd => 1,
      c_a_fraction_width => 53,
      c_latency => 56,
      c_has_rdy => 1,
      c_result_fraction_width => 53,
      c_has_divide => 1,
      c_rate => 56,
      c_has_operation_rfd => 1,
      c_result_width => 64,
      c_b_width => 64,
      c_a_width => 64)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      operation_rfd => rfd,
      clk => clk,
      result => result_slv,
      rdy => rdy);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  process (clk)
  begin
    if clk'event and clk = '1' then
      if (rdy = '1') then
        result <= std_ulogic_vector(result_slv);
      end if;
    end if;
  end process;
  
  ack0: entity work.impulse_acknowledge port map (clk, go, rdy, done);
  
end fdivd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fsqrtd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    done: out std_ulogic);
end fsqrtd_ll;

architecture fsqrtd_ll_a of fsqrtd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy, rfd : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fsqrt : floating_point_v2_0
    generic map(
      c_has_operation_nd => 1,
      c_a_fraction_width => 53,
      c_latency => 55,
      c_has_rdy => 1,
      c_result_fraction_width => 53,
      c_has_sqrt => 1,
      c_rate => 55,
      c_has_operation_rfd => 1,
      c_result_width => 64,
      c_a_width => 64)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      operation_rfd => rfd,
      clk => clk,
      result => result_slv,
      rdy => rdy);

  a_slv <= std_logic_vector(a);
  process (clk)
  begin
    if clk'event and clk = '1' then
      if (rdy = '1') then
        result <= std_ulogic_vector(result_slv);
      end if;
    end if;
  end process;
  
  ack0: entity work.impulse_acknowledge port map (clk, go, rdy, done);
  
end fsqrtd_ll_a;

library ieee;
use ieee.std_logic_1164.all;

entity fnegd_ll is
  port (
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(63 downto 0));
end fnegd_ll;

architecture fnegd_ll_a of fnegd_ll is
begin
  result(63) <= a(63) xor '1';
  result(62 downto 0) <= a(62 downto 0);
end fnegd_ll_a;

library ieee;
use ieee.std_logic_1164.all;

entity fabsd_ll is
  port (
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(63 downto 0));
end fabsd_ll;

architecture fabsd_ll_a of fabsd_ll is
begin
  result(63) <= '0';
  result(62 downto 0) <= a(62 downto 0);
end fabsd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity feqd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end feqd_ll;

architecture feqd_ll_a of feqd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_feq : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PT_EQUAL,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);
  
end feqd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fneqd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end fneqd_ll;

architecture fneqd_ll_a of fneqd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_fneq : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PT_NOT_EQUAL,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);
  
end fneqd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity fltd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end fltd_ll;

architecture fltd_ll_a of fltd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_flt : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PT_LESS_THAN,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);
  
end fltd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_consts.all;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity flteqd_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
end flteqd_ll;

architecture flteqd_ll_a of flteqd_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal b_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_flteq : floating_point_v2_0
    generic map(
      c_has_compare => 1,
      c_compare_operation => FLT_PRT_LESS_THAN_OR_EQUAL,
      c_a_width => 64,
      c_a_fraction_width => 53,
      c_b_width => 64,
      c_b_fraction_width => 53,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 3)
    port map (
      a => a_slv,
      b => b_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  b_slv <= std_logic_vector(b);
  result(0) <= result_slv(0);

end flteqd_ll_a;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity ftod_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(63 downto 0));
end ftod_ll;

architecture ftod_ll_a of ftod_ll is
    signal exp_s : std_ulogic_vector(7 downto 0);
    signal exp_d : unsigned(10 downto 0);
    signal zero : std_ulogic;
begin
  exp_s <= a(30 downto 23);             -- extract the exponent
  exp_d <= unsigned("000" & exp_s) + 896;  -- rebias
  zero <= '1' when a = X"00000000" else '0';

  result(63) <= a(31);                  -- sign bit
   -- biased exponent
  result(62 downto 52) <= std_ulogic_vector(exp_d) when zero = '0'
                          else "00000000000";
  result(51 downto 29) <= a(22 downto 0);  -- fraction
  result(28 downto 0) <= "00000000000000000000000000000";
end ftod_ll_a;
  
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity dtof_ll is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(31 downto 0));
end dtof_ll;

architecture dtof_ll_a of dtof_ll is
    signal exp_s : std_ulogic_vector(7 downto 0);
    signal exp_d : std_ulogic_vector(10 downto 0);
    signal exp : unsigned(10 downto 0);
    signal ovr, inf : std_ulogic;
begin
  exp_d <= a(62 downto 52);             -- extract the exponent
  exp <= unsigned(exp_d) - 896;  -- rebias
  exp_s <= std_ulogic_vector(exp(7 downto 0));
  ovr <= '0' when std_ulogic_vector(exp(10 downto 8)) = "000" else '1';
  inf <= a(62) when ovr = '1' else '0';

  -- sign bit
  result(31) <= a(63);
  -- exponent
  result(30 downto 23) <= exp_s when ovr = '0'
                          else "00000000" when inf = '0'
                          else "11111111";
  -- fraction
  result(22 downto 0) <= a(51 downto 29) when ovr = '0'
                         else "00000000000000000000000";
end dtof_ll_a;
  
library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity itof_ll is
  generic (
    IWIDTH : positive);
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(IWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
end itof_ll;

architecture itof_ll_a of itof_ll is
    signal a_slv: std_logic_vector(IWIDTH-1 downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_itof : floating_point_v2_0
    generic map(
      c_has_fix_to_flt => 1,
      c_a_width => IWIDTH,
      c_a_fraction_width => 0,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 2)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  result <= std_ulogic_vector(result_slv);
  
end itof_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity itod_ll is
  generic (
    IWIDTH : positive);
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(IWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
end itod_ll;

architecture itod_ll_a of itod_ll is
    signal a_slv: std_logic_vector(IWIDTH-1 downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_itod : floating_point_v2_0
    generic map(
      c_has_fix_to_flt => 1,
      c_a_width => IWIDTH,
      c_a_fraction_width => 0,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 2)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  result <= std_ulogic_vector(result_slv);
  
end itod_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity utof_ll is
  generic (
    UWIDTH : positive);
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(UWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
end utof_ll;

architecture utof_ll_a of utof_ll is
    signal a_slv: std_logic_vector(UWIDTH downto 0);
    signal result_slv: std_logic_vector(31 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_utof : floating_point_v2_0
    generic map(
      c_has_fix_to_flt => 1,
      c_a_width => UWIDTH+1,
      c_a_fraction_width => 0,
      c_result_fraction_width => 24,
      c_result_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 2)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= '0' & std_logic_vector(a);
  result <= std_ulogic_vector(result_slv);
  
end utof_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity utod_ll is
  generic (
    UWIDTH : positive);
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(UWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
end utod_ll;

architecture utod_ll_a of utod_ll is
    signal a_slv: std_logic_vector(UWIDTH downto 0);
    signal result_slv: std_logic_vector(63 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_utod : floating_point_v2_0
    generic map(
      c_has_fix_to_flt => 1,
      c_a_width => UWIDTH+1,
      c_a_fraction_width => 0,
      c_result_fraction_width => 53,
      c_result_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 2)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= '0' & std_logic_vector(a);
  result <= std_ulogic_vector(result_slv);
  
end utod_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity ftoi_ll is
  generic (
    IWIDTH : positive);
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(IWIDTH-1 downto 0);
    pipeEn: in std_ulogic);
end ftoi_ll;

architecture ftoi_ll_a of ftoi_ll is
    signal a_slv: std_logic_vector(31 downto 0);
    signal result_slv: std_logic_vector(IWIDTH-1 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_ftoi : floating_point_v2_0
    generic map(
      c_has_flt_to_fix => 1,
      c_result_width => IWIDTH,
      c_result_fraction_width => 0,
      c_a_fraction_width => 24,
      c_a_width => 32,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 1)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  result <= std_ulogic_vector(result_slv);
  
end ftoi_ll_a;

library ieee;
use ieee.std_logic_1164.all;
library XilinxCoreLib;
use XilinxCoreLib.floating_point_v2_0_comp.all;

entity dtoi_ll is
  generic (
    IWIDTH : positive);
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(IWIDTH-1 downto 0);
    pipeEn: in std_ulogic);
end dtoi_ll;

architecture dtoi_ll_a of dtoi_ll is
    signal a_slv: std_logic_vector(63 downto 0);
    signal result_slv: std_logic_vector(IWIDTH-1 downto 0);
    signal operation_nd, rdy : std_logic;
begin
  operation_nd <= go;
  
  xilinx_dtoi : floating_point_v2_0
    generic map(
      c_has_flt_to_fix => 1,
      c_result_width => IWIDTH,
      c_result_fraction_width => 0,
      c_a_fraction_width => 53,
      c_a_width => 64,
      c_has_ce=> 1,
      c_has_operation_nd => 1,
      c_rate => 1,
      c_latency => 2)
    port map (
      a => a_slv,
      operation_nd => operation_nd,
      clk => clk,
      result => result_slv,
      ce => pipeEn);

  a_slv <= std_logic_vector(a);
  result <= std_ulogic_vector(result_slv);
  
end dtoi_ll_a;


library ieee;
use ieee.std_logic_1164.all;

package xilinx_float_low_latency is
  component fadd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fsub_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fmul_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fdiv_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    done: out std_ulogic);
  end component;

  component fsqrt_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    done: out std_ulogic);
  end component;

  component fneg_ll is
    port (
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(31 downto 0));
  end component;

  component fabs_ll is
    port (
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(31 downto 0));
  end component;

  component feq_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fneq_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component flt_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component flteq_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    b: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component faddd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fsubd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fmuld_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fdivd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    done: out std_ulogic);
  end component;

  component fsqrtd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    done: out std_ulogic);
  end component;

  component fnegd_ll is
    port (
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(63 downto 0));
  end component;

  component fabsd_ll is
    port (
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(63 downto 0));
  end component;

  component feqd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fneqd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component fltd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component flteqd_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    b: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(0 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component ftod_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(63 downto 0));
  end component;

  component dtof_ll is
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(31 downto 0));
  end component;

  component itof_ll is
    generic (IWIDTH : positive);
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(IWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
  end component;
  
  component itod_ll is
    generic (IWIDTH : positive);
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(IWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
  end component;
  
  component utof_ll is
    generic (UWIDTH : positive);
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(UWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(31 downto 0);
    pipeEn: in std_ulogic);
  end component;
  
  component utod_ll is
    generic (UWIDTH : positive);
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(UWIDTH-1 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(63 downto 0);
    pipeEn: in std_ulogic);
  end component;

  component ftoi_ll is
    generic (IWIDTH : positive);
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(IWIDTH-1 downto 0);
    pipeEn: in std_ulogic);
  end component;
  
  component dtoi_ll is
    generic (IWIDTH : positive);
    port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    go: in std_ulogic;
    result: out std_ulogic_vector(IWIDTH-1 downto 0);
    pipeEn: in std_ulogic);
  end component;
  

end xilinx_float_low_latency;
