--------------------------------------------------------------------------
--
-- Copyright (c) 2005 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
--
-- This source file may be used and distributed without restriction
-- provided that this copyright statement is not removed from the file
-- and that any derivative work contains this copyright notice.
--
-- cray_rt_if.vhd: Modules for interfaces between Cray Rapid Array Transport
--     and Impulse C primitives (streams, signals, registers, shared memory)
--
-- Change History
-- --------------
-- 10/10/2005 - Ralph Bodenner
--   File created.
--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package cray_rt_if is
	component rt_to_stream is
		generic (
			datawidth : positive := 64
		);
		port (
			rt_reset_n : in std_logic;
			clk : in std_logic;
			rt_wr : in std_logic;
			rt_rd : in std_logic;
			rt_wdata : in std_logic_vector(63 downto 0);
			rt_rdata : out std_logic_vector(63 downto 0);
			rt_addr : in std_logic_vector(1 downto 0);
			rt_resp_complete : in std_logic;
			stream_en : out std_logic;
			stream_rdy : in std_logic;
			stream_eos : out std_logic;
			stream_data : out std_logic_vector (datawidth-1 downto 0)
		);
	end component;
	
	component stream_to_rt is 
		generic (
			datawidth : positive := 64
		);
		port (
			rt_reset_n : in std_logic;
			clk : in std_logic;
			rt_wr : in std_logic;
			rt_rd : in std_logic;
			rt_wdata : in std_logic_vector(63 downto 0);
			rt_rdata : out std_logic_vector(63 downto 0);
			rt_addr : in std_logic_vector(1 downto 0);
			rt_resp_complete : in std_logic;
			stream_en : out std_logic;
			stream_rdy : in std_logic;
			stream_eos : in std_logic;
			stream_data : in std_logic_vector (datawidth-1 downto 0)
		);
	end component;

	component rt_to_signal is
		generic (
			datawidth : positive := 32
		);
		port (
			rt_reset_n : in std_logic;
			clk : in std_logic;
			rt_wr : in std_logic;
			rt_rd : in std_logic;
			rt_wdata : in std_logic_vector(63 downto 0);
			rt_rdata : out std_logic_vector(63 downto 0);
			rt_addr : in std_logic_vector(1 downto 0);
			signal_en : out std_logic;
			signal_data : out std_logic_vector (datawidth-1 downto 0)
		);
	end component;

	component signal_to_rt is
		generic (
			datawidth : positive := 32
		);
		port (
			rt_reset_n : in std_logic;
			clk : in std_logic;
			rt_wr : in std_logic;
			rt_rd : in std_logic;
			rt_wdata : in std_logic_vector(63 downto 0);
			rt_rdata : out std_logic_vector(63 downto 0);
			rt_addr : in std_logic_vector(1 downto 0);
			signal_en : out std_logic;
			signal_rdy : in std_logic;
			signal_data : in std_logic_vector (datawidth-1 downto 0)
		);
	end component;
end package cray_rt_if;

library ieee;
use ieee.std_logic_1164.all;

entity rt_to_stream is
	generic (
		datawidth : positive := 64
	);
	port (
		rt_reset_n : in std_logic;
		clk : in std_logic;
		rt_wr : in std_logic;
		rt_rd : in std_logic;
		rt_wdata : in std_logic_vector(63 downto 0);
		rt_rdata : out std_logic_vector(63 downto 0);
		rt_addr : in std_logic_vector(1 downto 0);
		rt_resp_complete : in std_logic;
		stream_en : out std_logic;
		stream_rdy : in std_logic;
		stream_eos : out std_logic;
		stream_data : out std_logic_vector (datawidth-1 downto 0)
	);
end rt_to_stream;

architecture rtl of rt_to_stream is
  signal error, write_addr : std_logic;
  signal status : std_logic_vector (63 downto 0);
begin
  write_addr <=
    '1' when rt_addr = "00" else
    '1' when rt_addr = "10" else
    '0';

  -- Write to stream
  stream_en <= stream_rdy and rt_wr and write_addr;
  stream_eos <= '1' when rt_addr = "10" else '0';
  stream_data <= rt_wdata(datawidth-1 downto 0);

  -- Error detection
  check: process (rt_reset_n, clk)
  begin
    if (rt_reset_n = '0') then
      error <= '0';
    elsif (clk'event and clk='1') then
      if (rt_wr = '1' and write_addr = '1') then
        error <= not stream_rdy;
      end if;
    end if;
  end process;
  
  -- Status register
  status(0) <= stream_rdy;
  status(1) <= '0';
  status(2) <= '0';
  status(3) <= error;
  status(63 downto 4) <= "000000000000000000000000000000000000000000000000000000000000";

  rt_rdata <=
    status when rt_addr = "01" else
    X"0000000000000000";
end rtl;

library ieee;
use ieee.std_logic_1164.all;

entity stream_to_rt is 
	generic (
		datawidth : positive := 64
	);
	port (
		rt_reset_n : in std_logic;
		clk : in std_logic;
		rt_wr : in std_logic;
		rt_rd : in std_logic;
		rt_wdata : in std_logic_vector(63 downto 0);
		rt_rdata : out std_logic_vector(63 downto 0);
		rt_addr : in std_logic_vector(1 downto 0);
		rt_resp_complete : in std_logic;
		stream_en : out std_logic;
		stream_rdy : in std_logic;
		stream_eos : in std_logic;
		stream_data : in std_logic_vector (datawidth-1 downto 0)
	);
end stream_to_rt;

architecture rtl of stream_to_rt is
  signal accept, valid, eos, error, address_stream : std_logic;
  signal status : std_logic_vector (63 downto 0);
  signal data : std_logic_vector (63 downto 0);
  signal extended_data : std_logic_vector (63 downto 0);
begin
  address_stream <= '1' when rt_addr = "00" else '0';
  -- Read register
  accept <= (not valid or (rt_rd and address_stream and rt_resp_complete));
  extended_data(63 downto datawidth) <= (others => '0');
  extended_data(datawidth - 1 downto 0) <= stream_data;

  read_stream: process (rt_reset_n, clk)
  begin
    if (rt_reset_n = '0') then
      valid <= '0';
      eos <= '0';
    elsif (clk'event and clk='1') then
      if (stream_rdy = '1' and accept = '1') then
        data <= extended_data;
        eos <= stream_eos;
        valid <= '1';
      elsif (rt_rd = '1' and address_stream = '1' and rt_resp_complete = '1') then
        valid <= '0';
      end if;
    end if;
  end process;

  stream_en <= accept;
  
  -- Error detection
  check: process (rt_reset_n, clk)
  begin
  	if (rt_reset_n = '0') then
      error <= '0';
    elsif (clk'event and clk='1') then
      if (rt_rd = '1' and address_stream = '1' and rt_resp_complete = '1') then
        error <= not valid;
      end if;
    end if;
  end process;
  
  -- Status register
  status(0) <= valid;
  status(1) <= eos;
  status(2) <= '0';
  status(3) <= error;
  status(63 downto 4) <= "000000000000000000000000000000000000000000000000000000000000";

  rt_rdata <=
    data when rt_addr = "00" else
    status when rt_addr = "01" else
    X"0000000000000000";
end rtl;

library ieee;
use ieee.std_logic_1164.all;

entity rt_to_signal is
	generic (
		datawidth : positive := 32
	);
	port (
		rt_reset_n : in std_logic;
		clk : in std_logic;
		rt_wr : in std_logic;
		rt_rd : in std_logic;
		rt_wdata : in std_logic_vector(63 downto 0);
		rt_rdata : out std_logic_vector(63 downto 0);
		rt_addr : in std_logic_vector(1 downto 0);
		signal_en : out std_logic;
		signal_data : out std_logic_vector (datawidth-1 downto 0)
	);
end rt_to_signal;

architecture rtl of rt_to_signal is
	signal write_addr : std_ulogic;
	signal status : std_logic_vector (63 downto 0);
begin
	write_addr <= '1' when rt_addr = "00" else '0';

	-- Write to signal
	signal_en <= rt_wr and write_addr;
	signal_data <= std_logic_vector(rt_wdata(datawidth-1 downto 0));
 
	-- Status register
	status <= X"0000000000000001"; -- Always ready/no errors.

	rt_rdata <=
		status when rt_addr = "01" else
		X"0000000000000000";
end rtl;

library ieee;
use ieee.std_logic_1164.all;

entity signal_to_rt is
	generic (
		datawidth : positive := 32
	);
	port (
		rt_reset_n : in std_logic;
		clk : in std_logic;
		rt_wr : in std_logic;
		rt_rd : in std_logic;
		rt_wdata : in std_logic_vector(63 downto 0);
		rt_rdata : out std_logic_vector(63 downto 0);
		rt_addr : in std_logic_vector(1 downto 0);
		signal_en : out std_logic;
		signal_rdy : in std_logic;
		signal_data : in std_logic_vector (datawidth-1 downto 0)
	);
end signal_to_rt;

architecture rtl of signal_to_rt is
	signal accept, valid, error, address_signal : std_ulogic;
	signal status : std_logic_vector (63 downto 0);
	signal data : std_logic_vector (63 downto 0);
	signal extended_data : std_logic_vector (63 downto 0);
begin
	address_signal <= '1' when rt_addr = "00" else '0';
	-- Read register
	accept <= '1';
	extended_data(63 downto datawidth) <= (others => '0');
	extended_data(datawidth-1 downto 0) <= signal_data;

	read_signal: process (rt_reset_n, clk)
	begin
		if (rt_reset_n = '0') then
			valid <= '0';
		elsif (clk'event and clk='1') then
			if (signal_rdy = '1' and accept = '1') then
				data <= extended_data;
				valid <= '1';
			elsif (rt_rd = '1' and address_signal = '1') then
				valid <= '0';
			end if;
		end if;
	end process;

	signal_en <= accept;

	-- Error detection
	check: process (rt_reset_n, clk)
	begin
		if (rt_reset_n = '0') then
			error <= '0';
		elsif (clk'event and clk='1') then
			if (rt_rd = '1' and address_signal = '1') then
				error <= not valid;
			else
				error <= '0';
			end if;
		end if;
	end process;

	-- Status register
	status(0) <= valid;
	status(1) <= '0';
	status(2) <= '0';
	status(3) <= error;
	status(63 downto 4) <= (others => '0');

	rt_rdata <=
		std_logic_vector(data) when rt_addr = "00" else
		status when rt_addr = "01" else
		X"0000000000000000";
end rtl;

