///////////////////////////////////////////////////////////////////////////////
//
// Impulse-C(c) 2003-2008 Impulse Accelerated Technologies, Inc.
//
// FPWM2008i_sw.c: includes the software test bench processes and
// main() function for the parallel "fixed-point" version of the 
// FPWM system.
//
// See additional comments in FPWM2008i.h.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "co.h"
#include "cosim_log.h"
#include "FPWM2008i.h"

extern co_architecture co_initialize(void *);

// Globals
static char *pwmHeader, *dnaHeader;
static int *pwm;
static char *dnaSequence;
static int pwmSize, dnaLength;
static int filterMode;
static char* pwmList[2];


//
// This is the function for encoding a floating-point value to fixed-point,
// taken from the software framework of the existing VHDL solution
//
int fpEncode (float x) {
	return x * 16777216;
}


//
// This is the function for decoding a fixed-point value back to floating-point,
// taken from the software framework of the existing VHDL solution
//
float fpDecode(int x) {
	return (float) ((x)/16777216.0);
}


//
// This is the function for reading an input sequence from a file, based on a function
// from the Impulse CoDeveloper example project 'SmithWatermanSerial'
//
void read_sequence(const char *DnaFileName, FILE *dnaInFile) {
	char *buffer = (char*)malloc(32*sizeof(char));
	char *sequence, *header;
	int sequenceLength;
	int size = 32;
	int status = 0;
	int nonvalid;
	char character;

	// Opening sequence input file
	dnaInFile = fopen(DnaFileName, "r");
	if ( dnaInFile == NULL ) {
		fprintf(stderr, "Error opening sequence input file %s\n", DnaFileName);
		character = getc(stdin);
		exit(-1);
	}

	// Finding the size of header line
	status = fread(buffer, sizeof(char), 1, dnaInFile);

	// Making sure the sequence is in the correct input format
	if(buffer[0] != '>') {
		printf("Sequence in %s must be in FASTA format, starting with '>'\n", DnaFileName);
	}

	// Triming off non-valid characters before sequence
	buffer[0] = '\n';
	while(!(VALID_CHAR((int)buffer[0]))) {
		fread(buffer, sizeof(char), 1, dnaInFile);
	}

	// Finding the size of the sequence, after pruning the header line
	fscanf(dnaInFile, "%s", buffer);
	sequenceLength = 0;
	nonvalid = 0;
	while(fread(buffer, sizeof(char), 1, dnaInFile)) {
		if(VALID_CHAR((int)buffer[0]))
			sequenceLength++;
		if (!(VALID_CHAR((int)buffer[0])))
			nonvalid++;
	}

	// Closing sequence input file
	if(fclose(dnaInFile) != 0) {
		printf("Error closing sequence input file\n");
	}

	// Allocating memory to store the sequence and header line
	header = (char*)malloc(size*sizeof(char));
	sequence = (char*)malloc((sequenceLength + 1)*sizeof(char));

	// Reopening sequence input file
	dnaInFile = fopen(DnaFileName, "r");
	if( dnaInFile == NULL ){
		printf("Error opening sequence input file %s file\n", DnaFileName);
		exit(-1);
	}

	// Pruning '>'
	fread(buffer, sizeof(char), 1, dnaInFile);

	// Reading sequence name
	status = fscanf(dnaInFile, "%s", header);
	if(status <= 0 || status > size){
		printf("Error reading %s file header\n", DnaFileName);
		exit(-1);
	}

	size = 1; /* Why shouldn't this value be 0? Would cause an error */
	//Reading the sequence
	printf("\nReading sequence %s: ", header);
	while (fread(buffer, sizeof(char), 1, dnaInFile)) {
		if (VALID_CHAR((int)buffer[0])) {
			sequence[size++] = buffer[0];
		}
	}
	printf("done\n");
	if(size-1 != sequenceLength){
		printf("Error reading %s file sequence\n", DnaFileName);
		exit(-1);
	}

	// Closing sequence input file
	if(fclose(dnaInFile) != 0) {
		printf("Error closing sequence input file\n");
	}

	// Saving sequence data
	dnaHeader = header;
	dnaSequence = sequence;
	dnaLength = sequenceLength;

	free(buffer);
}


//
// This is the function for reading a filter threshold from file
//
int read_threshold(const char * FilterFileName, FILE *filterInFile) {
	char *header;
	int status = 0;
	int size = 32; /* size of header */
	float threshold;
	
	header = (char*)malloc(size*sizeof(char));

	// Reading matrix name
	status = fscanf(filterInFile, "%s", header);
	if(status <= 0 || status > size){
		printf("Error reading %s file header\n", FilterFileName);
		exit(-1);
	}
	
	// Reading threshold value
	fscanf(filterInFile, "%f", &threshold);

	free(header);

	// Returning threshold value
	return fpEncode(threshold);
}


//
// This is the function for calculating pwm-values from count-values
//
int convert_value(float c, float N) {
	double s = 0.25;
	double P = 0.25;
	double p, valueTemp;
	float pwmValueTemp;
	
	p = (double) ((c+s)/(N+(4*s)));
	valueTemp = (double) (p/P);
	pwmValueTemp = (float) log(valueTemp);
	
	return fpEncode(pwmValueTemp);
}


//
// This is the function for converting an alignment matrix to a pwm
//
int *convert_matrix (float * matrix) {
	int i, j;
	int *pwmTemp;
	double N;

	pwmTemp = (int*)malloc(ROWS*COLUMNS*sizeof(int));
	for (j = 0; j < ROWS; j++) {
		N = (matrix[j*COLUMNS+0]+matrix[j*COLUMNS+1]+matrix[j*COLUMNS+2]+matrix[j*COLUMNS+3]);
		for (i = 0; i < COLUMNS; i++) {
			pwmTemp[j*COLUMNS+i] = convert_value(matrix[j*COLUMNS+i], N);
		}
	}
	return pwmTemp;
}


//
// This is the function for reading an input matrix from a file and converting it to a pwm,
// based on a function from the Impulse CoDeveloper example project 'SmithWatermanSerial'
//
void read_matrix(const char * PwmFileName, FILE *pwmInFile) {
	char *buffer = (char*)malloc(32*sizeof(char));
	char *header;
	int status = 0;
	int size = 32; /* size of header */
	int matrixSize = COLUMNS*ROWS;
	float *matrix;
	int i, j;

	// Finding the size of the header line and pruning '>'
	status = fread(buffer, sizeof(char), 1, pwmInFile);

	// Making sure the next matrix have been found 
	while(buffer[0] != '>') {
		status = fread(buffer, sizeof(char), 1, pwmInFile);
	}

	// Allocating memory to store header line and matrix
	header = (char*)malloc(size*sizeof(char));
	matrix = (float*)malloc(matrixSize*sizeof(float));

	// Reading matrix name
	status = fscanf(pwmInFile, "%s", header);
	if(status <= 0 || status > size){
		printf("Error reading %s file header\n", PwmFileName);
		exit(-1);
	}

	// Reading and storing the matrix
	printf("\nReading matrix %s: ", header);
	for(j = 0; j < ROWS; j++) {
		for(i = 0; i < COLUMNS; i++) {
			fscanf(pwmInFile, "%f", &matrix[j*COLUMNS+i]);
		}
	}
	printf("done\n");

	// Saving pwm data
	pwmHeader = header;
	printf("Converting %s: ", header);
	pwm = convert_matrix(matrix);
	printf("done\n");
	pwmSize = matrixSize;

	free(matrix);
	free(buffer);
}


//
// This is the software 'reader' process
//
void Producer(co_stream threshold_0, co_stream threshold_1, co_stream pwm_0, co_stream pwm_1, co_stream iteration_0, 
	co_stream iteration_1, co_stream dna_0, co_stream dna_1, co_parameter filters)
{
	int c;
	IF_SIM(cosim_logwindow log = cosim_logwindow_create("Producer");)

	// Opening streams
	co_stream_open(threshold_0, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(threshold_1, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(pwm_0, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(pwm_1, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(iteration_0, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));
	co_stream_open(iteration_1, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));
	co_stream_open(dna_0, O_WRONLY, CHAR_TYPE);
	co_stream_open(dna_1, O_WRONLY, CHAR_TYPE);

	// Preparing to send thresholds
	int n;
	int nSampleThreshold;

	// Preparing to send matrices
	int i, j;
	int nSamplePwm;

	// Preparing to send sequence
	int k;
	char nSampleDna;

	// Reading sequence from input file
	const char * DnaFileName = DNA_INPUT_FILE;
	FILE * dnaInFile;
	read_sequence(DnaFileName, dnaInFile);

	// Determining type of filter based on command line argument
	if(filters == NULL) {
		// Filter is set to summation filter
		filterMode = 1;
	} else {
		// Filter is set to threshold filter
		filterMode = 2;
		
		// Opening threshold input file
		const char * FilterFileName = filters;
		FILE * filterInFile;
		filterInFile = fopen(FilterFileName, "r");
		if ( filterInFile == NULL ) {
			fprintf(stderr, "Error opening filter input file %s\n", FilterFileName);
			c = getc(stdin);
			exit(-1);
		}

		// Reading threshold value for first matrix from input file and sending it
		nSampleThreshold = read_threshold(FilterFileName, filterInFile);
		co_stream_write(threshold_0, &nSampleThreshold, sizeof(int32));
		// Reading threshold value for second matrix from input file and sending it
		nSampleThreshold = read_threshold(FilterFileName, filterInFile);
		co_stream_write(threshold_1, &nSampleThreshold, sizeof(int32));
		
		// Closing threshold input file
		if(fclose(filterInFile) != 0) {
			printf("Error closing filter input file\n");
		}
	}

	// Opening matrix input file
	const char * PwmFileName = PWM_INPUT_FILE;
	FILE * pwmInFile;
	pwmInFile = fopen(PwmFileName, "r");
	if ( pwmInFile == NULL ) {
		fprintf(stderr, "Error opening matrix input file %s\n", PwmFileName);
		c = getc(stdin);
		exit(-1);
	}

	// Reading first input matrix and converting it to pwm
	read_matrix(PwmFileName, pwmInFile);
	pwmList[0] = (char*)pwmHeader;
	
	// Sending first matrix
	printf("Sending %s: ", pwmList[0]);
	for(i = 0; i < pwmSize; i++) {
		nSamplePwm = (int)pwm[i];
		co_stream_write(pwm_0, &nSamplePwm, sizeof(int32));
	}
	printf("done\n");
	free(pwm);
	
	// Reading second input matrix and converting it to pwm
	read_matrix(PwmFileName, pwmInFile);
	pwmList[1] = (char*)pwmHeader;
	
	// Sending second matrix
	printf("Sending %s: ", pwmList[1]);
	for(i = 0; i < pwmSize; i++) {
		nSamplePwm = (int)pwm[i];
		co_stream_write(pwm_1, &nSamplePwm, sizeof(int32));
	}
	printf("done\n");
	free(pwm);

	// Closing matrix input file
	if(fclose(pwmInFile) != 0) {
		printf("Error closing matrix input file\n");
	}

	// Sending sequence length and actual sequence
	co_stream_write(iteration_0, &dnaLength, sizeof(int16));
	co_stream_write(iteration_1, &dnaLength, sizeof(int16));
	printf("\nSending %s...\n\n", dnaHeader);
	for(k = 1; k <= dnaLength; k++) {
		nSampleDna = (char)dnaSequence[k];
		co_stream_write(dna_0, &nSampleDna, sizeof(char));
		co_stream_write(dna_1, &nSampleDna, sizeof(char));
	}
	printf("done sending %s\n", dnaHeader);
	free(dnaSequence);

	// Closing streams
	co_stream_close(threshold_0);
	co_stream_close(threshold_1);
	co_stream_close(pwm_0);
	co_stream_close(pwm_1);
	co_stream_close(iteration_0);
	co_stream_close(iteration_1);
	co_stream_close(dna_0);
	co_stream_close(dna_1);
}


//
// This is the software 'writer' process
//
void Consumer(co_stream start_0, co_stream score_0, co_stream start_1, co_stream score_1)
{	
	int16 nResultStart;
	int resultEnd;
	int32 nResultScore;
	float resultScore;
	unsigned int count = 0;
  	const char * FileName = OUTPUT_FILE;
	FILE * outFile;

	IF_SIM(cosim_logwindow log = cosim_logwindow_create("Consumer");)

	// Opening output file
	outFile = fopen(FileName, "w");
	if ( outFile == NULL ) {
		fprintf(stderr, "Error opening file %s for writing\n", FileName);
		exit(-1);
	}
	
	// Opening streams
	co_stream_open(start_0, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
	co_stream_open(score_0, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(start_1, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
	co_stream_open(score_1, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/2));

	IF_SIM(cosim_logwindow_write(log, "Consumer reading results...\n");)
	
	// Reading filtered results streamed from first filter; then writing them to screen and file
	while (co_stream_read(start_0, &nResultStart, sizeof(int16)) == co_err_none) {
		if(co_stream_read(score_0, &nResultScore, sizeof(int32)) == co_err_none) {
			if(filterMode == 1) {	// Summation filter	
				resultEnd = dnaLength - 1;
			} else {	// Threshold filter
				resultEnd = nResultStart + (ROWS-1);
			}
			resultScore = fpDecode(nResultScore);
			fprintf(outFile, "%s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmList[0], resultScore);
			IF_SIM(cosim_logwindow_fwrite(log, "Result: %s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmList[0], resultScore);)
			printf("%s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmList[0], resultScore);
			count++;
		}
	}
	
	// Reading filtered results streamed from second filter; then writing them to screen and file
	while (co_stream_read(start_1, &nResultStart, sizeof(int16)) == co_err_none) {
		if(co_stream_read(score_1, &nResultScore, sizeof(int32)) == co_err_none) {
			if(filterMode == 1) {	// Summation filter
				resultEnd = dnaLength - 1;
			} else {	// Threshold filter
				resultEnd = nResultStart + (ROWS-1);
			}
			resultScore = fpDecode(nResultScore);
			fprintf(outFile, "%s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmList[1], resultScore);
			IF_SIM(cosim_logwindow_fwrite(log, "Result: %s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmList[1], resultScore);)
			printf("%s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmList[1], resultScore);
			count++;
		}
	}

	IF_SIM(cosim_logwindow_fwrite(log, "Consumer read %d filtered results\n", count);)
	printf("\n\nThe application produced %d filtered scores for %s\n", count, dnaHeader);

	free(dnaHeader);
	free(pwmHeader);

	// Closing output file
	if(fclose(outFile) != 0) {
		printf("Error closing result output file\n");
	}

	// Closing streams
	co_stream_close(start_0);
	co_stream_close(score_0);
	co_stream_close(start_1);
	co_stream_close(score_1);
}


//
// Impulse C Main Function
//
int main(int argc, char **argv)
{
	co_architecture my_arch;
	void *param = NULL; 
	char *filter;
	char **arg;
	int c;
	
	printf("\n\n== IMPULSE-C APPLICATION: FPWM 2008i ==\n");
	switch(argc) {
		case 1:
			printf("Executing with summation filter\n");
			my_arch = co_initialize(param);
			co_execute(my_arch);
			break;
		case 2:
			printf("Executing with threshold filter\n");
			arg = (char**)argv;
			filter = (char*)arg[1];
			my_arch = co_initialize(filter);
			co_execute(my_arch);
			break;
		default:
			printf("\nWrong use of parameters!\n");
			break;
	}

	printf("Application complete! Press the Enter key to continue...\n");
	c = getc(stdin);

	return(0);
}
