///////////////////////////////////////////////////////////////////////////////
//
// Impulse-C(c) 2003-2008 Impulse Accelerated Technologies, Inc.
//
// FPWMi_sw.c: includes the software test bench processes and
// main() function for the basic "fixed-point" version of the
// FPWM system.
//
// See additional comments in FPWMi.h.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "co.h"
#include "cosim_log.h"
#include "FPWMi.h"

extern co_architecture co_initialize(void *);

// Globals
static char *pwmHeader, *dnaHeader;
static int *pwm;
static char *dnaSequence;
static int pwmSize, dnaLength;
static int filterMode;


//
// This is the function for encoding a floating-point value to fixed-point,
// taken from the software framework of the existing VHDL solution
//
int fpEncode (float x) {
	return x * 16777216;
}


//
// This is the function for decoding a fixed-point value back to floating-point,
// taken from the software framework of the existing VHDL solution
//
float fpDecode(int x) {
	return (float) ((x)/16777216.0);
}


//
// This is the function for calculating pwm-values from count-values
//
int convert_value(float c, float N) {
	double s = 0.25;
	double P = 0.25;
	double p, valueTemp;
	float pwmValueTemp;
	
	p = (double) ((c+s)/(N+(4*s)));
	valueTemp = (double) (p/P);
	pwmValueTemp = (float) log(valueTemp);

	return fpEncode(pwmValueTemp);
}


//
// This is the function for converting an alignment matrix to a pwm
//
int *convert_matrix (float * matrix) {
	int i, j;
	int *pwmTemp;
	double N;

	pwmTemp = (int*)malloc(ROWS*COLUMNS*sizeof(int));
	for (j = 0; j < ROWS; j++) {
		N = (matrix[j*COLUMNS+0]+matrix[j*COLUMNS+1]+matrix[j*COLUMNS+2]+matrix[j*COLUMNS+3]);
		for (i = 0; i < COLUMNS; i++) {
			pwmTemp[j*COLUMNS+i] = convert_value(matrix[j*COLUMNS+i], N);
		}
	}
	return pwmTemp;
}


//
// This is the function for reading an input matrix from a file and converting it to a pwm,
// based on a function from the Impulse CoDeveloper example project 'SmithWatermanSerial'
//
void read_matrix(const char * PwmFileName, FILE *pwmInFile) {
	char *buffer = (char*)malloc(32*sizeof(char));
	char *header;
	int status = 0;
	int size = 32; /* size of header */
	int matrixSize = COLUMNS*ROWS;
	float *matrix;
	int i, j; 
	char character;

	// Opening matrix input file
	pwmInFile = fopen(PwmFileName, "r");
	if ( pwmInFile == NULL ) {
		fprintf(stderr, "Error opening matrix input file %s\n", PwmFileName);
		character = getc(stdin);
		exit(-1);
	}

	// Finding the size of the header line
	status = fread(buffer, sizeof(char), 1, pwmInFile);

	// Making sure the matrix is in the correct input format
	if(buffer[0] != '>') {
		printf("Matrix in %s must be in FASTA format, starting with '>'\n", PwmFileName);
	}

	// Closing matrix input file
	if(fclose(pwmInFile) != 0) {
		printf("Error closing matrix input file\n");
	}

	// Allocating memory to store header line
	header = (char*)malloc(size*sizeof(char));
	matrix = (float*)malloc(matrixSize*sizeof(float));

	// Reopening matrix input file
	pwmInFile = fopen(PwmFileName, "r");
	if( pwmInFile == NULL ){
		printf("Error opening matrix input file %s file\n", PwmFileName);
		exit(-1);
	}

	// Pruning '>'
	fread(buffer, sizeof(char), 1, pwmInFile);

	// Reading matrix name
	status = fscanf(pwmInFile, "%s", header);
	if(status <= 0 || status > size){
		printf("Error reading %s file header\n", PwmFileName);
		exit(-1);
	}

	// Reading and storing the matrix
	printf("\nReading matrix %s: ", header);
	for(j = 0; j < ROWS; j++) {
		for(i = 0; i < COLUMNS; i++) {
			fscanf(pwmInFile, "%f", &matrix[j*COLUMNS+i]);
		}
	}
	printf("done\n");

	// Closing matrix input file
	if(fclose(pwmInFile) != 0) {
		printf("Error closing matrix input file\n");
	}

	// Saving pwm data
	pwmHeader = header;
	printf("Converting %s: ", header);
	pwm = convert_matrix(matrix);
	printf("done\n");
	pwmSize = matrixSize;

	free(matrix);
	free(buffer);
}


//
// This is the function for reading an input sequence from a file, based on a function
// from the Impulse CoDeveloper example project 'SmithWatermanSerial'
//
void read_sequence(const char *DnaFileName, FILE *dnaInFile) {
	char *buffer = (char*)malloc(32*sizeof(char));
	char *sequence, *header;
	int sequenceLength;
	int size = 32;
	int status = 0;
	int nonvalid;
	char character;

	// Opening sequence input file
	dnaInFile = fopen(DnaFileName, "r");
	if ( dnaInFile == NULL ) {
		fprintf(stderr, "Error opening sequence input file %s\n", DnaFileName);
		character = getc(stdin);
		exit(-1);
	}
	// Finding the size of header line
	status = fread(buffer, sizeof(char), 1, dnaInFile);

	// Making sure the sequence is in the correct input format
	if(buffer[0] != '>') {
		printf("Sequence in %s must be in FASTA format, starting with '>'\n", DnaFileName);
	}

	// Triming off non-valid characters before sequence
	buffer[0] = '\n';
	while(!(VALID_CHAR((int)buffer[0]))) {
		fread(buffer, sizeof(char), 1, dnaInFile);
	}

	// Finding the size of the sequence, after pruning the header line
	fscanf(dnaInFile, "%s", buffer);
	sequenceLength = 0;
	nonvalid = 0;
	while(fread(buffer, sizeof(char), 1, dnaInFile)) {
		if(VALID_CHAR((int)buffer[0]))
			sequenceLength++;
		if (!(VALID_CHAR((int)buffer[0])))
			nonvalid++;
	}
	
	// Closing sequence input file
	if(fclose(dnaInFile) != 0) {
		printf("Error closing sequence input file\n");
	}

	// Allocating memory to store the sequence and header line
	header = (char*)malloc(size*sizeof(char));
	sequence = (char*)malloc((sequenceLength + 1)*sizeof(char));

	// Reopening sequence input file
	dnaInFile = fopen(DnaFileName, "r");
	if( dnaInFile == NULL ){
		printf("Error opening sequence input file %s file\n", DnaFileName);
		exit(-1);
	}

	// Pruning '>'
	fread(buffer, sizeof(char), 1, dnaInFile);

	// Reading sequence name
	status = fscanf(dnaInFile, "%s", header);
	if(status <= 0 || status > size){
		printf("Error reading %s file header\n", DnaFileName);
		exit(-1);
	}

	size = 1; /* Why shouldn't this value be 0? Would cause an error */
	//Reading the sequence
	printf("\nReading sequence %s: ", header);
	while (fread(buffer, sizeof(char), 1, dnaInFile)) {
		if (VALID_CHAR((int)buffer[0])) {
			sequence[size++] = buffer[0];
		}
	}
	printf("done\n");
	if(size-1 != sequenceLength){
		printf("Error reading %s file sequence\n", DnaFileName);
		exit(-1);
	}

	// Closing sequence input file
	if(fclose(dnaInFile) != 0) {
		printf("Error closing sequence input file\n");
	}

	// Save sequence data
	dnaHeader = header;
	dnaSequence = sequence;
	dnaLength = sequenceLength;

	free(buffer);
}


//
// This is the software 'reader' process
//
void Producer(co_stream threshold_stream, co_stream pwm_stream, co_stream iteration_stream, co_stream dna_stream, co_parameter filter)
{
	float threshold;
	IF_SIM(cosim_logwindow log = cosim_logwindow_create("Producer");)

	// Opening streams
	co_stream_open(threshold_stream, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(pwm_stream, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/2));
	co_stream_open(dna_stream, O_WRONLY, CHAR_TYPE);
	co_stream_open(iteration_stream, O_WRONLY, INT_TYPE(MAX_STREAMWIDTH/4));

	// Determining type of filter based on command line argument
	if(filter == NULL) {
		// Filter is set to summation filter
		filterMode = 1;
	} else {
		// Filter is set to threshold filter
		filterMode = 2;
		threshold = fpEncode(atof(filter));
		co_stream_write(threshold_stream, &threshold, sizeof(int32));
	}

	// Reading matrix from input file and converting it to pwm
	const char * PwmFileName = PWM_INPUT_FILE;
	FILE * pwmInFile;
	read_matrix(PwmFileName, pwmInFile);

	// Sending matrix
	int i, j;
	int nSamplePwm;
	printf("Sending %s: ", pwmHeader);
	for(i = 0; i < pwmSize; i++) {
		nSamplePwm = (int)pwm[i];
		co_stream_write(pwm_stream, &nSamplePwm, sizeof(int32));
	}
	printf("done\n");

	free(pwm);

	// Reading sequence from input file
	const char * DnaFileName = DNA_INPUT_FILE;
	FILE * dnaInFile;
	read_sequence(DnaFileName, dnaInFile);

	// Sending sequence length and actual sequence
	int k;
	char nSampleDna;
	co_stream_write(iteration_stream, &dnaLength, sizeof(int16));
	printf("Sending %s...\n\n", dnaHeader);
	for(k = 1; k <= dnaLength; k++) {
		nSampleDna = (char)dnaSequence[k];
		co_stream_write(dna_stream, &nSampleDna, sizeof(char));
	}
	printf("done sending %s\n", dnaHeader);

	free(dnaSequence);

	// Closing streams
	co_stream_close(threshold_stream);
	co_stream_close(pwm_stream);
	co_stream_close(iteration_stream);
	co_stream_close(dna_stream);
}


//
// This is the software 'writer' process
//
void Consumer(co_stream start_stream, co_stream score_stream)
{	
	int16 nResultStart;
	int resultEnd;
	int32 nResultScore;
	float resultScore;
	unsigned int count = 0;
  	const char * FileName = OUTPUT_FILE;
	FILE * outFile;

	IF_SIM(cosim_logwindow log = cosim_logwindow_create("Consumer");)

	// Opening output file
	outFile = fopen(FileName, "w");
	if ( outFile == NULL ) {
		fprintf(stderr, "Error opening file %s for writing\n", FileName);
		exit(-1);
	}
	
	// Opening streams
	co_stream_open(start_stream, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/4));
	co_stream_open(score_stream, O_RDONLY, INT_TYPE(MAX_STREAMWIDTH/2));

	IF_SIM(cosim_logwindow_write(log, "Consumer reading results...\n");)
	
	// Reading filtered results from stream; then writing them to screen and file
	while (co_stream_read(start_stream, &nResultStart, sizeof(int16)) == co_err_none) {
		if(co_stream_read(score_stream, &nResultScore, sizeof(int32)) == co_err_none) {
			if(filterMode == 1) {	// Summation filter
				resultEnd = dnaLength - 1;
			} else {	// Threshold filter
				resultEnd = nResultStart + (ROWS-1);
			}
			resultScore = fpDecode(nResultScore);
			fprintf(outFile, "%s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmHeader, resultScore);
			IF_SIM(cosim_logwindow_fwrite(log, "Result: %s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmHeader, resultScore);)
			printf("%s %d %d %s %f\n", dnaHeader, nResultStart, resultEnd, pwmHeader, resultScore);
			count++;
		}
	}
	IF_SIM(cosim_logwindow_fwrite(log, "Consumer read %d filtered results\n", count);)
	printf("\n\nThe application produced %d filtered scores for %s\n", count, dnaHeader);

	free(dnaHeader);
	free(pwmHeader);

	// Closing output file
	if(fclose(outFile) != 0) {
		printf("Error closing result output file\n");
	}

	// Closing streams
	co_stream_close(start_stream);
	co_stream_close(score_stream);
}


//
// Impulse C Main Function
//
int main(int argc, char **argv)
{
	co_architecture my_arch;
	void *param = NULL; 
	char *filter;
	char **arg;
	int c;
	
	printf("\n\n== IMPULSE-C APPLICATION: FPWMi ==\n");
	switch(argc) {
		case 1:
			printf("Executing with summation filter\n");
			my_arch = co_initialize(param);
			co_execute(my_arch);
			break;
		case 2:
			printf("Executing with threshold filter; ");
			arg = (char**)argv;
			filter = (char*)arg[1];
			printf("threshold set at %s\n", filter);
			my_arch = co_initialize(filter);
			co_execute(my_arch);
			break;
		default:
			printf("\nWrong use of parameters!\n");
			break;
	}

	printf("Application complete! Press the Enter key to continue...\n");
	c = getc(stdin);

	return(0);
}
